/* Measures CO2 Level of the air and indicates it with colours
 *  Data Sheet of CO2-Sensor MH-Z19B:
 *  https://www.winsen-sensor.com/d/files/infrared-gas-sensor/mh-z19b-co2-ver1_0.pdf
 *  
 *  with Code from the NEOPIXEL library examples (function "rainbowCycle" and function "Wheel")
 *  
 *  by Sebastian Voss
 *  
 *  You are free to use, distribute and modify this software at your own risk.
 *  
 *  Difference to the advanced version: Usage of the Adafruit ST7735 library instead of
 *  the deprecated TFT library
 */

#include <Wire.h>
#include <cactus_io_BME280_I2C.h>
#include <Adafruit_NeoPixel.h>
#include <Adafruit_GFX.h>     // Core graphics library
#include <Adafruit_ST7735.h>  // Hardware-specific library
#include <SPI.h>

// pin definition for the Uno
#define cs   10
#define rs   9
#define res  8

Adafruit_ST7735 screen = Adafruit_ST7735(cs, rs, res);

#define CO2_PWM 4
#define NEOPIXEL_PIN 2
#define NEOPIXEL_NUM 24
#define BRIGHTNESS 20

unsigned long preheatTime = 3*60*1000L;    // sensor needs 3min for heating up
int range = 5000;               // sensor measures from 0ppm to 5000ppm

long ppm = 0;                // variable for storing the ppm value

String co2value;

Adafruit_NeoPixel strip = Adafruit_NeoPixel(NEOPIXEL_NUM, NEOPIXEL_PIN, NEO_GRB + NEO_KHZ800);

bool BME_ENABLED = true;

uint32_t pres;                        // Luftdruck
uint32_t hum;                         // relative Luftfeuchtigkeit
int8_t temp;                          // Temperatur

BME280_I2C bme(0x76);  // I2C using address 0x76


int interval = 10000;            // time between two measurements in ms

void setup() {
  Serial.begin(9600);
  pinMode(CO2_PWM, INPUT);

  // start Neopixels and set them to 'off'
  strip.begin();
  strip.setBrightness(BRIGHTNESS);
  strip.show();

  // starting the screen
  screen.initR(INITR_BLACKTAB);   // use INITR_GREENTAB if red and blue are exchanged and if there is coloured border
  screen.setRotation(1);
  screen.fillScreen(ST77XX_BLACK);
  delay(500);
  screen.setTextColor(ST77XX_WHITE, ST77XX_BLACK);  // Textcolour and Colour of textbackground (second is optional)
  screen.setTextSize(2);
  screen.setCursor(10,10);
  screen.print("AirBuddy");
  drawLogo(128,5);
  
  delay(100);

  if (BME_ENABLED) {
    // start bme280
    if (!bme.begin()) {
      Serial.println("BME280 nicht gestartet");
    } else {
      Serial.println("BME280 gestartet");
    }
    bme.setTempCal(-5);   // offset, because temperature sensor is near to voltage regulator which produces heat  
  }
  
  
  Serial.println("Testing MH-Z19B with PWM Interface");
  
  Serial.println("Preheating (3min)");

  screen.setTextSize(1);
  screen.setTextColor(ST77XX_WHITE, ST77XX_BLACK);  
  // Textcolour and Colour of textbackground
  // Colour of background is used for overwriting former texts and values
  
  screen.setCursor(5,60);
  screen.print("Sensor wird aufgeheizt...");
  String timeLeft = "";
  while (millis() < preheatTime) {
    Serial.println("... (Time running: " + String(millis()/1000) + " s)");

    if (BME_ENABLED) {
      // read bme values
      bme.readSensor();
      temp=bme.getTemperature_C();       // Bme280 - Temperature in celsius
      pres=bme.getPressure_MB();         // Bme280 - Pressure in milliBar / hectoPascal
      hum=bme.getHumidity();             // Bme280 - relative humidity
    
      displayBMEValues(temp, pres, hum);
    }
    
    // show time left
    timeLeft = "Noch " + String(180 - millis()/1000) + "s...";
    screen.setCursor(5,80);
    screen.print(timeLeft);

    // rainbow animation while waiting for preheating
    // duration of rainbow approx 5s with 20 as argument (see below)
    rainbowCycle(20);
  }
  Serial.println();
  Serial.println("Start!");
  // clear screen and add heading again
  screen.fillScreen(ST77XX_BLACK);
  delay(500);
  screen.setTextColor(ST77XX_WHITE, ST77XX_BLACK);  // Textcolour and Colour of textbackground (second is optional)
  screen.setTextSize(2);
  screen.setCursor(10,10);
  screen.print("AirBuddy");
  drawLogo(128,5);
}

void loop() {
  Serial.print("\n----- Time from start: ");
  Serial.print(millis() / 1000);
  Serial.println(" s");

  if(BME_ENABLED) {
    // read bme values
    bme.readSensor();
    temp=bme.getTemperature_C();       // Bme280 - Temperature in celsius
    pres=bme.getPressure_MB();         // Bme280 - Pressure in milliBar / hectoPascal
    hum=bme.getHumidity();             // Bme280 - relative humidity
  
    // display new values from bme
    displayBMEValues(temp, pres, hum);
  }
  
  // measure co2 level
  ppm = getCO2();
  
  if ( ppm == -1) {    // handle error of measurement
    
    // output on serial monitor
    Serial.println("Measure failed, now pausing, then restarting");

    // output on display
    screen.fillScreen(ST77XX_BLACK);
    screen.setTextSize(2);
    screen.setCursor(10, 10);
    screen.print("AirBuddy");
    drawLogo(128,5);
    screen.setTextSize(1);
    screen.setTextColor(ST77XX_WHITE, ST77XX_BLACK);
    screen.setCursor(5, 40);
    screen.print("Messfehler. Messung wird ");
    screen.setCursor(5, 60);
    screen.print("nach Pause wiederholt.");

    // output on neopixels
    strip.setPixelColor(0, strip.Color( 255,   0,   255,   150));
    for(int i=1; i< NEOPIXEL_NUM; i++) {
      strip.setPixelColor(i, strip.Color(0,0,0,0));
    }
    strip.show();

    // wait for the time left, when subtracting the measurement time
    delay( interval - 10 * interval / (2*10) );
    
    // clear screen and add heading again
    screen.fillScreen(ST77XX_BLACK);
    screen.setTextSize(2);
    screen.setCursor(10, 10);
    screen.print("AirBuddy");
    drawLogo(128,5);

    // skip next steps of this loop turn
    return;
  }

  // display co2 level on the screen
  displayCO2Value(ppm);

  // visualize co2 level with neopixels
  visualizePPM(ppm);

  // output of co2 level on serial monitor
  Serial.println("CO2 value: " + String(ppm) + "ppm");
  Serial.println();
  Serial.println();

  // wait
  delay(interval);
}

/* function displays the new values of the bme280 sensor and deletes the former output
 * by overwriting the old values
 */

void displayBMEValues(int8_t t, uint32_t p, uint32_t h) {
  
  String tempValue = String(t) + "C";
  if ( -10 < t && t < 10 ) tempValue += " "; // add blank space for overwriting last character of former values, if necessary
  
  String presValue = String(p) + "hPa";
  if ( p < 1000 ) presValue += " "; // add blank space for overwriting last character of former values, if necessary
  
  String humValue = String(h) + "%";
  if ( h < 10) humValue += " "; // add blank space for overwriting last character of former values, if necessary

  // ouput on Serial monitor
  Serial.println("Temperature: " + tempValue);
  Serial.println("Air pressure: " + presValue);
  Serial.println("Humidity: " + humValue);

  // output on screen
  screen.setTextColor(ST77XX_WHITE, ST77XX_BLACK);
  screen.setTextSize(1);
  
  // temperature symbol
  screen.fillCircle(10, 46, 2, ST77XX_RED);
  screen.fillRect(9,38,2,7, ST77XX_RED);
  screen.fillRect(9,40,1,1, ST77XX_BLACK);
  screen.fillRect(9,42,1,1, ST77XX_BLACK);
  screen.fillRect(9,44,1,1, ST77XX_BLACK);

  // temperature value
  screen.setCursor(18, 40);
  screen.print(tempValue);

  // humidity symbol
  screen.fillCircle(55, 46, 1, ST77XX_BLUE);
  screen.fillCircle(57, 43, 1, ST77XX_BLUE);
  screen.fillCircle(53, 41, 1, ST77XX_BLUE);
  
  // humidity value
  screen.setCursor(63,40);
  screen.print(humValue);

  // pressure symbol
  screen.drawLine(98, 46, 101, 40, ST77XX_CYAN);
  screen.drawLine(101, 40, 104, 46, ST77XX_CYAN);
  screen.drawLine(104, 46, 102, 46, ST77XX_CYAN);
  
  // air pressure value
  screen.setCursor(108,40);
  screen.print(presValue);
  
}

/* function displays the ppm value (nppm)
 * 
 */
void displayCO2Value(long nppm) {
  
  co2value = String(nppm) + "ppm";
  if ( nppm < 1000 ) co2value += " "; // add blank space for overwriting last character of former values, if necessary

  screen.setTextColor(ST77XX_WHITE, ST77XX_BLACK);
  screen.setTextSize(3);
  screen.setCursor(15, 60);
  screen.print(co2value);

  // explain new ppm value
  screen.setTextColor(ST77XX_WHITE, ST77XX_BLACK);
  screen.setTextSize(1);
  if ( nppm < 500 ) {
    screen.setCursor(5, 100);
    screen.print("Die Luft ist frisch      ");
    screen.setCursor(5, 115);
    screen.print("wie draussen!            ");
  }
  else if ( nppm < 750 ) {
    screen.setCursor(5, 100);
    screen.print("Gesunde Raumluft.        ");
  }
  else if ( nppm < 1000 ) {
    screen.setCursor(5, 100);
    screen.print("Akzeptable Raumluft.     ");
  }
  else if ( nppm < 1500 ) {
    screen.setCursor(5, 100);
    screen.print("Voruebergehend akzeptable");
    screen.setCursor(5, 115);
    screen.print("Raumluft. Bitte lueften! ");
  }
  else if ( nppm < 2000 ) {
    screen.setCursor(5, 100);
    screen.print("Eher ungesunde Raumluft. ");
    screen.setCursor(5, 115);
    screen.print("Bitte lueften!           ");
  }
  else {
    screen.setCursor(5, 100);
    screen.print("Ungesunde Raumluft.      ");
    screen.setCursor(5, 115);
    screen.print("Sofort lueften!          ");
  }
  
}

/* function for drawing the logo on the screen
 * 
 */
void drawLogo(int xpos, int ypos) {
  int w = 1; // width of pixel
  int r = 0; // row
  // first row
  screen.fillRect(xpos+6,ypos + r,w,w, ST77XX_ORANGE);
  screen.fillRect(xpos+17,ypos + r,w,w, ST77XX_ORANGE);
  r++;
  
  // second row
  screen.fillRect(xpos+5, ypos + r, 3*w, w, ST77XX_ORANGE);
  screen.fillRect(xpos+16, ypos + r, 3*w, w, ST77XX_ORANGE);
  r++;
  
  // third row
  screen.fillRect(xpos+4,ypos + r, 2*w, w, ST77XX_ORANGE);
  screen.fillRect(xpos + 6, ypos + r, w, w, 0xAAAA);
  screen.fillRect(xpos+7,ypos + r, 2*w, w, ST77XX_ORANGE);

  screen.fillRect(xpos+15,ypos + r, 2*w, w, ST77XX_ORANGE);
  screen.fillRect(xpos + 17, ypos + r, w, w, 0xAAAA);
  screen.fillRect(xpos+18,ypos + r, 2*w, w, ST77XX_ORANGE);
  r++;

  // row 4
  screen.fillRect(xpos+4,ypos + r, 2*w, w, ST77XX_ORANGE);
  screen.fillRect(xpos + 6, ypos + r, w, w, 0xAAAA);
  screen.fillRect(xpos+7,ypos + r, 2*w, w, ST77XX_ORANGE);

  screen.fillRect(xpos+15,ypos + r, 2*w, w, ST77XX_ORANGE);
  screen.fillRect(xpos + 17, ypos + r, w, w, 0xAAAA);
  screen.fillRect(xpos+18,ypos + r, 2*w, w, ST77XX_ORANGE);
  r++;

  // row 5
  screen.fillRect(xpos+3,ypos + r, 2*w, w, ST77XX_ORANGE);
  screen.fillRect(xpos + 5, ypos + r, 3*w, w, 0xAAAA);
  screen.fillRect(xpos+8,ypos + r, 2*w, w, ST77XX_ORANGE);

  screen.fillRect(xpos+14,ypos + r, 2*w, w, ST77XX_ORANGE);
  screen.fillRect(xpos + 16, ypos + r, 3*w, w, 0xAAAA);
  screen.fillRect(xpos+19,ypos + r, 2*w, w, ST77XX_ORANGE);
  r++;
  
  // row 6
  screen.fillRect(xpos+3,ypos + r, 2*w, w, ST77XX_ORANGE);
  screen.fillRect(xpos + 5, ypos + r, 3*w, w, 0xAAAA);
  screen.fillRect(xpos+8,ypos + r, 2*w, w, ST77XX_ORANGE);

  screen.fillRect(xpos+14,ypos + r, 2*w, w, ST77XX_ORANGE);
  screen.fillRect(xpos + 16, ypos + r, 3*w, w, 0xAAAA);
  screen.fillRect(xpos+19,ypos + r, 2*w, w, ST77XX_ORANGE);
  r++;

  // row 7
  screen.fillRect(xpos+2,ypos + r, 2*w, w, ST77XX_ORANGE);
  screen.fillRect(xpos + 4, ypos + r, 2*w, w, 0xAAAA);
  screen.fillRect(xpos + 6, ypos + r, 1*w, w, 0x7777);
  screen.fillRect(xpos + 7, ypos + r, 2*w, w, 0xAAAA);
  screen.fillRect(xpos+9,ypos + r, 2*w, w, ST77XX_ORANGE);

  screen.fillRect(xpos+13,ypos + r, 2*w, w, ST77XX_ORANGE);
  screen.fillRect(xpos + 15, ypos + r, 2*w, w, 0xAAAA);
  screen.fillRect(xpos + 17, ypos + r, 1*w, w, 0x7777);
  screen.fillRect(xpos + 18, ypos + r, 2*w, w, 0xAAAA);
  screen.fillRect(xpos+20,ypos + r, 2*w, w, ST77XX_ORANGE);
  r++;

  // row 8
  screen.fillRect(xpos+2,ypos + r, 2*w, w, ST77XX_ORANGE);
  screen.fillRect(xpos + 4, ypos + r, 2*w, w, 0xAAAA);
  screen.fillRect(xpos + 6, ypos + r, 1*w, w, 0x7777);
  screen.fillRect(xpos + 7, ypos + r, 2*w, w, 0xAAAA);
  screen.fillRect(xpos+9,ypos + r, 2*w, w, ST77XX_ORANGE);

  screen.fillRect(xpos+13,ypos + r, 2*w, w, ST77XX_ORANGE);
  screen.fillRect(xpos + 15, ypos + r, 2*w, w, 0xAAAA);
  screen.fillRect(xpos + 17, ypos + r, 1*w, w, 0x7777);
  screen.fillRect(xpos + 18, ypos + r, 2*w, w, 0xAAAA);
  screen.fillRect(xpos+20,ypos + r, 2*w, w, ST77XX_ORANGE);
  r++;

  // row 9
  screen.fillRect(xpos+1,ypos + r, 22*w, w, ST77XX_ORANGE);
  r++;

  // row 10
  screen.fillRect(xpos+0,ypos + r, 24*w, w, ST77XX_ORANGE);
  r++;

  // row 11
  screen.fillRect(xpos+0,ypos + r, 24*w, w, ST77XX_ORANGE);
  screen.fillRect(xpos+6,ypos + r, 2*w, w, ST77XX_BLACK);
  screen.fillRect(xpos+16,ypos + r, 2*w, w, ST77XX_BLACK);
  r++;

  // row 12
  screen.fillRect(xpos+1,ypos + r, 22*w, w, ST77XX_ORANGE);
  screen.fillRect(xpos+5,ypos + r, 4*w, w, ST77XX_BLACK);
  screen.fillRect(xpos+15,ypos + r, 4*w, w, ST77XX_BLACK);
  r++;

  // row 13
  screen.fillRect(xpos+2,ypos + r, 20*w, w, ST77XX_ORANGE);
  screen.fillRect(xpos+10,ypos + r, 1*w, w, ST77XX_ORANGE);  // darker?
  screen.fillRect(xpos+13,ypos + r, 1*w, w, ST77XX_ORANGE);  // darker?
  r++;

  // row 14
  screen.fillRect(xpos+2,ypos + r, 20*w, w, ST77XX_ORANGE);
  screen.fillRect(xpos+8,ypos + r, 2*w, w, ST77XX_ORANGE);  // darker?
  screen.fillRect(xpos+14,ypos + r, 2*w, w, ST77XX_ORANGE); // darker?
  r++;

  // row 15
  screen.fillRect(xpos+3,ypos + r, 1*w, w, ST77XX_WHITE);
  screen.fillRect(xpos+20,ypos + r, 1*w, w, ST77XX_WHITE);
  screen.fillRect(xpos+4,ypos + r, 16*w, w, ST77XX_ORANGE);
  screen.fillRect(xpos+9,ypos + r, 1*w, w, ST77XX_RED);
  screen.fillRect(xpos+15,ypos + r, 1*w, w, ST77XX_RED);
  r++;

  // row 16
  screen.fillRect(xpos+3,ypos + r, 1*w, w, ST77XX_WHITE);
  screen.fillRect(xpos+20,ypos + r, 1*w, w, ST77XX_WHITE);
  screen.fillRect(xpos+4,ypos + r, 16*w, w, ST77XX_ORANGE);
  screen.fillRect(xpos+9,ypos + r, 1*w, w, ST77XX_RED);
  screen.fillRect(xpos+15,ypos + r, 1*w, w, ST77XX_RED);
  r++;

  // row 17
  screen.fillRect(xpos+3,ypos + r, 3*w, w, ST77XX_WHITE);
  screen.fillRect(xpos+18,ypos + r, 3*w, w, ST77XX_WHITE);
  screen.fillRect(xpos+6,ypos + r, 12*w, w, ST77XX_ORANGE);
  r++;

  // row 18
  screen.fillRect(xpos+5,ypos + r, 3*w, w, ST77XX_WHITE);
  screen.fillRect(xpos+16,ypos + r, 3*w, w, ST77XX_WHITE);
  screen.fillRect(xpos+6,ypos + r, 12*w, w, ST77XX_ORANGE);
  screen.fillRect(xpos+10,ypos + r, 1*w, w, ST77XX_RED);
  screen.fillRect(xpos+13,ypos + r, 1*w, w, ST77XX_RED);
  r++;

  // row 19
  screen.fillRect(xpos+5,ypos + r, 3*w, w, ST77XX_WHITE);
  screen.fillRect(xpos+16,ypos + r, 3*w, w, ST77XX_WHITE);
  screen.fillRect(xpos+8,ypos + r, 8*w, w, ST77XX_ORANGE);
  screen.fillRect(xpos+10,ypos + r, 1*w, w, ST77XX_RED);
  screen.fillRect(xpos+13,ypos + r, 1*w, w, ST77XX_RED);
  r++;

  // row 20
  screen.fillRect(xpos+7,ypos + r, 1*w, w, ST77XX_WHITE);
  screen.fillRect(xpos+16,ypos + r, 1*w, w, ST77XX_WHITE);
  screen.fillRect(xpos+8,ypos + r, 8*w, w, ST77XX_ORANGE);
  screen.fillRect(xpos + 11, ypos + r, 2*w, w, ST77XX_BLACK);
  r++;

  // row 21
  screen.fillRect(xpos+7,ypos + r, 2*w, w, ST77XX_WHITE);
  screen.fillRect(xpos+15,ypos + r, 2*w, w, ST77XX_WHITE);
  screen.fillRect(xpos+9,ypos + r, 6*w, w, ST77XX_ORANGE);
  screen.fillRect(xpos + 11, ypos + r, 2*w, w, ST77XX_BLACK);
  r++;

  // row 22
  screen.fillRect(xpos+8,ypos + r, 1*w, w, ST77XX_WHITE);
  screen.fillRect(xpos+15,ypos + r, 1*w, w, ST77XX_WHITE);
  screen.fillRect(xpos+9,ypos + r, 6*w, w, ST77XX_ORANGE);
  r++;

  // row 23
  screen.fillRect(xpos+8,ypos + r, 8*w, w, ST77XX_WHITE);
  screen.fillRect(xpos+11,ypos + r, 2*w, w, ST77XX_ORANGE);
  r++;

  // row 24
  screen.fillRect(xpos+10,ypos + r, 4*w, w, ST77XX_WHITE);
  
}

/* Measure CO2 in ppm by analyzing the pwm-pin according to p. 6 in the data sheet (see header for link)
 * returns -1, if measure didn't succeed
 */
long getCO2(void) {
  
  long th, tl, ppm_pwm = 0;

  byte counter = 5;         // try to measure five times, otherwise report an error
  do {
    th = pulseIn(CO2_PWM, HIGH, 1004000) / 1000;
    if (th == 0) {
      Serial.println("Measure failed.");
      counter--;
    }
    tl = 1004 - th;
    ppm_pwm = range * (th - 2) / (th + tl - 4);
  } while (th == 0 && counter > 0);

  if ( counter == 0) return -1;

  ppm_pwm = doStatisticsOn(ppm_pwm);
  
  return ppm_pwm;
}

/* Since the MH-Z19B seems not to measure consistent values, when considering several sensors,
 *  here are done some statistics, which will be different for each sensor. It is assumed, that
 *  the arithmetical mean of two sensors is a better value, than the measured value, so the measured
 *  value is recalculated to fit the mean value.
 * 
 */
long doStatisticsOn(long ppm) {

  return ppm; // if you can't compare your sensor to another sensor you should leave this value as it is

//  if ( ppm < 550 ) return ppm;
//  return min(long(2.02856 * float(ppm) - 575.42315), 5000); // regression on sensor nr. 1
  
}


/* Colour Coding for air quality:
 *  ppm < 500: Fresh air - Blue,
 *  500 < ppm < 750: Healthy - DarkGreen, 
 *  750 < ppm < 1000: Acceptable - LightGreen,
 *  1000 < ppm < 1500: temporary acceptable - Yellow,
 *  1500 < ppm < 2000: rather unhealthy - Orange, 
 *  > 2000: really unhealthy - DarkRed
 *  ppm ==-1 means an error, which is coded with a blinking LED
 */

void visualizePPM(long ppm_pwm) {

  strip.setBrightness(BRIGHTNESS);
  uint32_t farbe;

  if ( ppm_pwm == -1) {
    farbe = strip.Color( 255,   0,   255,   150); 
    strip.setPixelColor(0, farbe);
    for(int i=1; i< NEOPIXEL_NUM; i++) {
      strip.setPixelColor(i, strip.Color(0,0,0,0));
    }
    strip.show();
    return;
  }
  else if (ppm_pwm < 500) {
    farbe = strip.Color(  0,   0,  255, 150); // Blue
  }
  else if (ppm_pwm < 750) {
    farbe = strip.Color(  0, 200,  0, 150); // DarkGreen
  }
  else if ( ppm_pwm < 1000) {
    farbe = strip.Color(  100, 190,  0, 150); // LightGreen
  }
  else if ( ppm_pwm < 1500) {
    farbe = strip.Color( 255, 255,   0, 150); // Yellow
  }
  else if ( ppm_pwm < 2000) {
    farbe = strip.Color( 255, 100,  0, 150); // orange
  }
  else {
    farbe = strip.Color( 204,   0,   0, 150); // DarkRed
  }

  for(int i=0; i< NEOPIXEL_NUM; i++) {
    strip.setPixelColor(i, farbe);
  }        
  strip.show();
}

// Rainbow animation of pixels
// duration of one cycle with delay of 20ms: 256*20ms=5120ms=5,12s > approx. 5s
void rainbowCycle(uint8_t wait) {
  uint16_t i, j;

  for(j=0; j<256*1; j++) { // 1 cycle of all colors on wheel
    for(i=0; i< strip.numPixels(); i++) {
      strip.setPixelColor(i, Wheel(((i * 256 / strip.numPixels()) + j) & 255));
    }
    strip.show();
    delay(wait);
  }
}

// Input a value 0 to 255 to get a color value.
// The colours are a transition r - g - b - back to r.
uint32_t Wheel(byte WheelPos) {
  WheelPos = 255 - WheelPos;
  if(WheelPos < 85) {
    return strip.Color(255 - WheelPos * 3, 0, WheelPos * 3);
  }
  if(WheelPos < 170) {
    WheelPos -= 85;
    return strip.Color(0, WheelPos * 3, 255 - WheelPos * 3);
  }
  WheelPos -= 170;
  return strip.Color(WheelPos * 3, 255 - WheelPos * 3, 0);
}
